#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include <netinet/in.h>
#include <string.h>
#include <stdlib.h>
#include <netdb.h>

static bool debug=false;

static void n_debug (const char *ctl, ...)
{
	if (debug){
		va_list list;
		va_start (list,ctl);
		vfprintf (stderr,ctl,list);
		va_end (list);
	}
}



static FILE *findprint_opendefprinter(const char *key)
{
	char var[PATH_MAX];	
	sprintf (var,"/var/xterminals/%s/sysconfig/DEFPRINTER",key);
	return fopen (var,"r");
}

/*
	Get the mac address from an IP number.
	This assumes the IP was used very recently, which is
	the case at login time.
*/
static int findprint_getmac (const char *ip, char mac[20])
{
	int ret = -1;
	FILE *fin = fopen ("/proc/net/arp","r");
	if (fin != NULL){
		char buf[1000];
		while (fgets (buf,sizeof(buf)-1,fin)!=NULL){
			char a[10][100];
			if (sscanf(buf,"%s %s %s %s %s",a[0],a[1],a[2],a[3],a[4])>=4){
				n_debug ("getmac: Compare %s %s\n",ip,a[0]);
				if (strcmp(a[0],ip)==0){
					strcpy (mac,a[3]);
					n_debug ("getmac: found %s\n",mac);
					ret = 0;
					break;
				}
			}
		}
		fclose (fin);
	}
	return ret;
}

static void findprint_main (const char *ip)
{
	n_debug ("The IP of the terminal is %s\n",ip);
	FILE *fin = findprint_opendefprinter(ip);
	if (fin == NULL){
		n_debug ("No terminal configured using the IP %s as the key\n",ip);
		// Ok, the terminal was configured using a MAC address
		char mac[20];
		if (findprint_getmac (ip,mac)!=-1){
			fin = findprint_opendefprinter(mac);
			if (fin == NULL){
				n_debug ("No terminal configured using MAC %s as the key\n",mac);
			}
		}
	}
	if (fin != NULL){
		char print[100];
		if (fgets(print,sizeof(print)-1,fin)!=NULL){
			printf ("export PRINTER=%s",print);
		}
		fclose (fin);
	}else{
		n_debug ("No xterminal configured for this IP\n");
	}
}


int main (int argc, char *argv[])
{
	int ret = -1;
	if (argc >= 2 && strcmp(argv[1],"--debug")==0){
		debug = true;
	}
	const char *disp = getenv ("DISPLAY");
	if (disp != NULL){
		char tmp[strlen(disp)+1];
		strcpy (tmp,disp);
		char *pt = strchr(tmp,':');
		if (pt != NULL){
			*pt = '\0';
			if (isalpha(tmp[0])){
				struct hostent *h = gethostbyname (tmp);
				if (h != NULL){
					unsigned long addr = ntohl (*(unsigned long*)h->h_addr);
					char ip[20];
					sprintf (ip,"%d.%d.%d.%d"
						,addr>>24
						,(addr>>16)&0xff
						,(addr>>8)&0xff
						,addr & 0xff);
					findprint_main (ip);
				}
			}else{
				// This is an IP number ?
				findprint_main (tmp);
			}
		}
	}
	return ret;
}

